from wbcrm.viewsets.titles.activities import (
    ActivityChartModelTitleConfig,
    ActivityParticipantTitleConfig,
    ActivityTitleConfig,
    ActivityTypeTitleConfig,
)

from .accounts import (
    AccountRoleAccountTitleConfig,
    AccountTitleConfig,
    ChildAccountAccountTitleConfig,
)
from .products import ProductTitleConfig
