# Generated by Django 5.0.9 on 2024-12-03 08:41

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0014_alter_account_relationship_status"),
    ]

    operations = [
        migrations.AlterField(
            model_name="activity",
            name="type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="activity",
                to="wbcrm.activitytype",
                verbose_name="Type",
            ),
        ),
    ]
