# Generated by Django 5.0.9 on 2024-11-19 14:11

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0013_account_action_plan_account_relationship_status_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="relationship_status",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="The Relationship Status from 1 to 5. 1 being the cold and 5 being the hot.",
                null=True,
                validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(5)],
                verbose_name="Relationship Status",
            ),
        ),
    ]
